#!/usr/bin/env perl

use strict;
use warnings;
sub usage {
    print "\nUsage:   3dna.pl [-options] <PDBfile>\n";
    print "Options: [-show all|none|out|inp|key1:keyn]\n";
    print "         [-inp name]\n";
    print "         [-setinp name]\n";
    print "         [-type]\n";
    print "         [-ref name [offset]]\n";
    print "         [-madbend]\n";
    print "         [-axis]\n";
    print "         [-groove minor|major|all]\n";
    print "         [-par shear|stretch|stagger|buckle|propel|open|all]\n";
    print "         [-step shift|slide|rise|tilt|roll|twist|all]\n";
    print "         [-helix x|y|hrise|incl|tip|htwist|all]\n";
    print "         [-pucker ampl|phase|both|pucker|all]\n";
    print "         [-torsion alpha|beta|gamma|delta|epsilon|zeta|chi|all]\n";
    print "         [-out name]\n";
    print "\n\n";
    exit 1;
    #####################################################################
    #                                                                   #
    # Options:                                                          #
    # -show                                                             #
    #    all - will keep all 3dna output                                #
    #    none - is the default                                          #
    #    out - will keep all output ending with "out" produced by 3dna  #
    #    inp - will keep all output ending with "inp" produeced by 3dna #
    #    key1:keyn - are user specified and will be kept                #
    #       example -show par:dat                                       #
    #       This will keep all 3dna output that contain "par" or "dat   #
    #       ANYWHERE in the file name.                                  #
    #                                                                   #
    # -inp name                                                         #
    #    Provide the name of an input file to be used for 3dna.         #
    #    This option is most useful for analyzing trajectories as the   #
    #    pdbfile will be different for every run and the first line of  #
    #    the 3dna input file needs to be adjusted accordingly.  Here,   #
    #    the input name that is provided will be copied and the first   #
    #    line (pdbfile name) will be altered to match the pdbfile from  #
    #    this command.                                                  #
    #                                                                   #
    # -setinp name                                                      #
    #    Allows user to specify the name of the input file that is      #
    #    created by 3dna (find_pair) rather than a generic $$.inp file  #
    #                                                                   #
    # -form                                                             #
    #    Identified A/B/TA-like (intermediate of A and B) dinucleotides #
    #                                                                   #
    # -ref name [offset]                                                #
    #    Generates output to the user specified file that gives you a   #
    #    reference for the analyses file output.                        #
    #    The offset value may be valuable when using this script with   #
    #    processDCD.pl which adds two additional columns to your        #
    #    output file.  In which case, set the offset to "2" so that the #
    #    reported columns in your reference file matches up with your   #
    #    output file.                                                   #
    #                                                                   #
    # -madbend                                                          #
    #    Generates the input parameter file to be read into madbend to  #
    #    calculate the global bending angle                             #
    #                                                                   #
    # -axis                                                             #
    #    Returns the global helical axis vector which can be used to    #
    #    calculate the angle between two vectors (acos(v1 dot v2))      #
    #                                                                   #
    # -groove parameter                                                 #
    #    Extract "refined minor/major groove values" from 3dna output.  #
    #    Note that it was suggested "as a rule" to always use the       #
    #    refined values!                                                #
    #                                                                   #
    # -par parameter                                                    #
    #    Extract "Local base-pair parameters" from 3dna output          #
    #                                                                   #
    # -step parameter                                                   #
    #    Extract "Local base-pair step parameters" from 3dna output     #
    #                                                                   #
    # -helix parameter                                                  #
    #    Extract "Local base-pair helical parameters" from 3dna output  #
    #                                                                   #
    # -pucker parameter                                                 #
    #    Gives pseudorotation information                               #
    #                                                                   #
    #####################################################################
}

#Check version
my $cmdv=`find_pair 2>&1`;
open(PIPE,"find_pair 2>&1 |");
my $version=1.5;
while (<PIPE>){
    $version=2 if ($_ =~ /v2.0/);
}
close(PIPE);

my $PDBfile;
my $show="none";
my @results=();
my @files=('auxiliary.par','bestpairs.pdb','bp_helical.par','bp_order.dat','bp_step.par','cf_7methods.par','col_chains.scr','col_helices.scr','hel_regions.pdb','hstacking.pdb','poc_haxis.r3d','ref_frames.dat','stacking.pdb');
my $tmpname;
my $inp;
my $groove;
my $step;
my $par;
my $helix;
my $ref;
my $offset=0;
my $form=0;
my $torsion;
my $pucker;
my %param=(
	   minor => '4', #Use the refined minor groove!
	   major => '6', #Use the refined major groove!
	   shear => '3',
	   stretch => '4',
	   stagger => '5',
	   buckle => '6',
	   propel => '7',
	   open => '8',
	   shift => '3',
	   slide => '4',
	   rise => '5',
	   tilt => '6',
	   roll => '7',
	   twist => '8',
	   x => '3',
	   y => '4',
	   hrise => '5',
	   incl => '6',
	   tip => '7',
	   htwist => '8',
	   alpha => '3',
	   beta => '4',
	   gamma => '5',
	   delta => '6',
	   epsilon => '7',
	   zeta => '8',
	   chi => '9',
	   ampl => '8',
	   phase => '9',
	   pucker => '10',
	   all => '0'
);
my @pall=();
my @sall=();
my @hall=();
my @tall=();
my %bpref=();
my %stack=();
my %strand=();
my @xref=();
my @keys=();
my $setinp;
my $madbend=0;
my @mref=();
my @tilt=();
my @roll=();
my @twist=();
my $axis=0;
my $out=undef;

while ($#ARGV>=0){
    if ($ARGV[0] eq "-h" || $ARGV[0] eq "-help" || $ARGV[0] eq "-"){
	&usage();
    }
    elsif ($ARGV[0] eq "-option"){
	##DO SOMETHING
    }
    elsif ($ARGV[0] eq "-show"){
	shift @ARGV;
	$show=shift @ARGV;
	@keys=split/\:/,$show;
    }
    elsif ($ARGV[0] eq "-inp"){
	shift @ARGV;
	$inp=shift @ARGV;
    }
    elsif ($ARGV[0] eq "-setinp"){
	shift @ARGV;
	$setinp=shift @ARGV;
    }
    elsif ($ARGV[0] eq "-ref"){
	shift @ARGV;
	$ref=shift @ARGV;
	if ($ARGV[0] =~ /^\d+$/){
	    $offset=shift @ARGV;
	    printf STDERR "\nOffset = $offset\n\n";
	}
    }
    elsif ($ARGV[0] eq "-offset"){
	shift @ARGV;
	$offset=shift @ARGV;
    }
    elsif ($ARGV[0] eq "-step"){
	shift @ARGV;
	$step=shift @ARGV;
	if (!defined $param{$step} || $step !~ /shift|slide|rise|tilt|roll|twist|all/){
	    die "\nError: Unknown option \"$step\" in -step\n";
	}
	elsif ($step eq "all"){
	    push(@sall,"shift");
	    push(@sall,"slide");
	    push(@sall,"rise");
	    push(@sall,"tilt");
	    push(@sall,"roll");
	    push(@sall,"twist");
	}
    }
    elsif ($ARGV[0] eq "-madbend"){
	shift @ARGV;
	$madbend=1;
	if ($show eq "none"){
	    $show="bp_step.par";
	}
	push(@keys, "bp_step.par");
    }
    elsif ($ARGV[0] eq "-axis"){
	shift @ARGV;
	$axis=1;
    }
    elsif ($ARGV[0] eq "-groove"){
	shift @ARGV;
	$groove=shift @ARGV;
	if (!defined $param{$groove} && $groove ne "all"){
	    die "\n\nError: Unknown option \"$groove\" in -groove\n\n";
	}
    }
    elsif ($ARGV[0] eq "-par"){
	shift @ARGV;
	$par=shift @ARGV;
	if (!defined $param{$par} || $par !~ /shear|stretch|stagger|buckle|propel|open|all/){
	    die "\n\nError: Unknown option \"$par\" in -par\n\n";
	}
	elsif($par eq "all"){
	    push(@pall,"shear");
	    push(@pall,"stretch");
	    push(@pall,"stagger");
	    push(@pall,"buckle");
	    push(@pall,"propel");
	    push(@pall,"open");
	}
    }
    elsif ($ARGV[0] eq "-helix"){
	shift @ARGV;
	$helix=shift @ARGV;
	if (!defined $param{$helix} || $helix !~ /x|y|hrise|incl|tip|htwist|all/){
	    die "\nError: Unknown option \"$helix\" in -helix\n";
	}
	elsif($helix eq "all"){
	    push(@hall,"x");
	    push(@hall,"y");
	    push(@hall,"hrise");
	    push(@hall,"incl");
	    push(@hall,"tip");
	    push(@hall,"htwist");
	}
    }
    elsif ($ARGV[0] eq "-form"){
	shift @ARGV;
	$form=1;
    }
    elsif ($ARGV[0] eq "-torsion"){
	shift @ARGV;
	$torsion=shift @ARGV;
	if ($torsion eq "all"){
	    push(@tall,"alpha");
	    push(@tall,"beta");
	    push(@tall,"gamma");
	    push(@tall,"delta");
	    push(@tall,"epsilon");
	    push(@tall,"zeta");
	    push(@tall,"chi");
	}
    }
    elsif ($ARGV[0] eq "-pucker"){
	shift @ARGV;
	$pucker=shift @ARGV;
    }
    elsif ($ARGV[0] eq "-out"){
	shift @ARGV;
	$out=shift @ARGV;
    }
    else {
	$PDBfile=shift @ARGV;
	my $tag=$PDBfile;
	if ($PDBfile =~ /\//){
	    #Remove path to directory
	    my @s=split/\//,$PDBfile;
	    $tag=$s[$#s];
	}
	my @tokens=split/\./,$tag;
	if ($#tokens==0){
	    $tmpname="$tokens[0]\.out";
	    push(@files,$tmpname);
	}
	else{
	    $tmpname="$tokens[0]";
	    for(my $i=1; $i<$#tokens;$i++){
		$tmpname=$tmpname."\.$tokens[$i]";
	    }
	    $tmpname=$tmpname.".out";
	    push (@files,$tmpname);
	}
    }
}


# If the input is being passed from another program
# write to a pdb file and delete it at the end
if (!defined $PDBfile){
    $PDBfile="3dna.pdb";
    push (@files,"3dna.pdb");
    open(PDB,">$PDBfile");
    while(<STDIN>){
	print PDB "$_";
    }
    close(PDB);
    my @tokens=split/\./,$PDBfile;
    if ($#tokens==0){
	$tmpname="$tokens[0]\.out";
	push(@files,$tmpname);
    }
    else{
	$tmpname="$tokens[0]";
	for(my $i=1; $i<$#tokens;$i++){
	    $tmpname=$tmpname."\.$tokens[$i]";
	}
	$tmpname=$tmpname.".out";
	push (@files,$tmpname);
    }
}


if (defined $inp){
    #Note: A copy of INP is changed and First line is changed to match PDBfile
    my $inpcp="$$.inp";
    open(INPUT,"$inp")|| die "\nError: Cannot open input file \"$inp\"\n\n";
    open(INPUTCP,">$inpcp");
    my $line=0;
    while(<INPUT>){
      $line++;
	if ($line==1){
	    if (chomp($_) ne $PDBfile){
		print INPUTCP "$PDBfile\n";
	    }
	}
  elsif($line==2){
    if (chomp($_) ne $tmpname){
    print INPUTCP "$tmpname\n";
    }
  }
	else{
	    print INPUTCP $_;
	}
    }
    close(INPUTCP);
    close(INPUT);
    system "analyze -c $$.inp";
    &reference();
    &parse();
    push(@files,$inpcp);
}
else{
    if (defined $setinp){
	$inp="$setinp";
    }
    else {
	$inp="$$.inp";
    }
    
    system "find\_pair $PDBfile $inp";
    system "analyze -c $inp";
   
    &reference();
    &parse();
    if ($madbend){
	$step="tilt";
	&parse();
	$step="roll";
	&parse();
	$step="twist";
	&parse();
	&madbend();
    }
    push(@files,$inp);
}

if ($show eq "none"){
    foreach (@files){
	unlink("$_");
    }
}
elsif ($show eq "out"){
    foreach (@files){
	if ($_ !~ m/\.out/){
	    unlink("$_");
	}
    }
}
elsif ($show eq "inp"){
    foreach (@files){
	if ($_ !~ m/\.inp/){
	    unlink("$_");
	}
    }
}
elsif ($show eq "all"){
    #DO NOTHING
}
elsif (defined $show && $#keys >= 0){
    for (my $j=0;$j<=$#files;$j++){
	foreach my $key (@keys){
	    if (defined $key && $files[$j] && $files[$j] =~ m/$key/){
		delete($files[$j]);
	    }
	}
    }
    foreach (@files){
	if (defined $_){
	    unlink ("$_");
	}
    }
}

####################
#                  #
#   Subroutines    #
#                  #
####################

#########################
#                       #
#    Output Reference   #
#                       #
#########################
sub reference{
    open(REF,"$inp");
    my $bpcount=1;
    my $stackcount=1;
    my @bpairs=();
    my @lastline=();
    my $linecount=1;
    my $nstrands;
    if ($version == 1.5){ #Version 1.5
	while (<REF>){
	    if ($_=~/\s+(\d+)\s+(\d+)\s+(\d+)\s+\#\s+(\d+)\s+.\s+(.\:\.+)(\d+)\_\:\[...\]\w\-*(\**)\-*\w\[...\]\:\.+(\d+)/ ){
		@bpairs= ("", $6, $8, $7);
		$bpref{$bpcount}="BP$bpcount:$6\-$8$7";
		if(defined $nstrands && $nstrands == 2){
		    $strand{I}{$bpcount}=$bpairs[1];
		    $strand{II}{$bpcount}=$bpairs[2];
		}
		$bpcount++;
	    }
	    elsif($linecount==3){
		my @strandline=split/\s+/,$_;
		$nstrands=$strandline[1];
		if ($nstrands ne 2){
		    die "\n\nError: This script was written for duplex and needs to be modified for other strand combinations.\n\n";
		}
	    }
	    if($bpcount==2){
		@lastline=@bpairs;
	    }
	    elsif($bpcount>2 && $_=~/\s+(\d+)\s+(\d+)\s+(\d+)\s+\#\s+(\d+)\s+.\s+(.\:\.+)(\d+)\_\:\[...\]\w\-*(\**)\-*\w\[...\]\:\.+(\d+)/){
		$stack{$stackcount}="$lastline[1]-$lastline[2]$lastline[3]/$bpairs[1]-$bpairs[2]$bpairs[3]";
		@lastline=@bpairs;
		$stackcount++;
	    }
	    $linecount++;
	}
    }
    elsif ($version == 2){ #Version 2.0
	while (<REF>){
	    if ($_=~/\s+(\d+)\s+(\d+)\s+(\d+)\s+\#\s+(\d+)\s+.\s+(\.+>\w+:\.+)(\d+)_\:\[...\]\w\-*(\**)\-*\w\[...\]\:\.+(\d+)/){
		@bpairs= ("", $6, $8, $7);
		$bpref{$bpcount}="BP$bpcount:$6\-$8$7";
		if(defined $nstrands && $nstrands == 2){
		    $strand{I}{$bpcount}=$bpairs[1];
		    $strand{II}{$bpcount}=$bpairs[2];
		}
		$bpcount++;
	    }
	    elsif($linecount==3){
		my @strandline=split/\s+/,$_;
		$nstrands=$strandline[1];
		if ($nstrands ne 2){
		    die "\n\nError: This script was written for duplex and needs to be modified for other strand combinations.\n\n";
		}
	    }
	    if($bpcount==2){
		@lastline=@bpairs;
	    }
	    elsif($bpcount>2 && $_=~/\s+(\d+)\s+(\d+)\s+(\d+)\s+\#\s+(\d+)\s+.\s+(\.+>\w+:\.+)(\d+)_\:\[...\]\w\-*(\**)\-*\w\[...\]\:\.+(\d+)/){
		$stack{$stackcount}="$lastline[1]-$lastline[2]$lastline[3]/$bpairs[1]-$bpairs[2]$bpairs[3]";
		@lastline=@bpairs;
		$stackcount++;
	    }
	    $linecount++;
	}
    }
    close(REF);
}

##########################
#                        #
#      Parse Output      #
#                        #
##########################

sub parse{
    open(RESULTS,"$tmpname");
    while(<RESULTS>){
	my @words=split/\s+/,$_;
	if (0){
	    #Add here if you need to read above step,par, and helix
	    #You may need to append the reference arrays!
	}
	elsif (defined $words[0] && $words[0] eq "Local"){
	    if ($words[2] eq "parameters" && defined $par){
		my $prm;
		if ($par ne "all"){
		    push (@results,$par);
		    push (@xref,$par);
		}
		else{
		    foreach $prm (@pall){
			push (@results,$prm);
			push (@xref,$prm);
		    }
		}
		while(<RESULTS>){
		    if($_ =~ m/\~+/){
			last;
		    }
		    elsif ($par eq "all"){
			my @lines=split/\s+/,$_;
			foreach $prm (@pall){
			    if ($lines[1] ne "bp"){
				push (@results,$lines[$param{$prm}]);
				push (@xref,$bpref{$lines[1]});
			    }
			}
		    }
		    else{
			my @lines=split/\s+/,$_;
			if ($lines[1] ne "bp"){
			    push (@results,$lines[$param{$par}]);
			    push (@xref,$bpref{$lines[1]});
			}
		    }
		}
	    }
	    elsif ($words[2] eq "step" && defined $step){
		my $stp;
		if ($step ne "all"){
		    push (@results,$step);
		    push (@xref,$step);
		}
		else{
		    foreach $stp (@sall){
			push (@results,$stp);
			push (@xref,$stp);
		    }
		}
		while(<RESULTS>){
		    if($_ =~ m/\~+/){
			last;
		    }
		    elsif ($step eq "all"){
			my @lines=split/\s+/,$_;
			if ($lines[1] ne "step"){
			    foreach $stp (@sall){
				push (@results,$lines[$param{$stp}]);
				push (@xref,$stack{$lines[1]});
			    }
			}
		    }
		    else{
			my @lines=split/\s+/,$_;
			if ($lines[1] ne "step"){
			    push (@results,$lines[$param{$step}]);
			    push (@xref,$stack{$lines[1]});
			    if ($madbend){
				#if ($step eq "tilt"){
				#    push (@tilt,$lines[$param{$step}]);
				#}
				#elsif ($step eq "roll"){
				#    push (@roll,$lines[$param{$step}]);
				#}
				#elsif ($step eq "twist"){
				#    push (@twist,$lines[$param{$step}]);
				#}
			    }
			}
		    }
		}
	    }
	    elsif ($words[2] eq "helical" && defined $helix){
		my $hlx;
		if ($helix ne "all"){
		    push (@results,$helix);
		    push (@xref,$helix);
		}
		else{
		    foreach $hlx (@hall){
			push (@results,$hlx);
			push (@xref,$hlx);
		    }
		}
		while(<RESULTS>){
		    if($_ =~ m/\~+/){
			last;
		    }
		    elsif ($helix eq "all"){
			my @lines=split/\s+/,$_;
			if ($lines[1] ne "step"){
			    foreach $hlx (@hall){
				push (@results,$lines[$param{$hlx}]);
				push (@xref,$stack{$lines[1]});
			    }
			}
		    }
		    else{
			my @lines=split/\s+/,$_;
			if ($lines[1] ne "step"){
			    push (@results,$lines[$param{$helix}]);
			    push (@xref,$stack{$lines[1]});
			}
		    }
		}
	    }
	}
	elsif (defined $words[0] && $words[0] eq "Helix\:"){
	    if ($axis){
		push (@results, $words[1]);
		push (@results, $words[2]);
		push (@results, $words[3]);
	    }
	}
	elsif(defined $words[0] && $words[0] eq "Classification"){
	    if ($form == 1){
		push (@results,"form");
		push (@xref,"form");
		while(<RESULTS>){
		    if($_ =~ m/\*+/){
			last;
		    }
		    else{
			my @lines=split/\s+/,$_;
			if (defined $lines[9] && $lines[0] ne "structure\:"){
			    push (@results,$lines[9]);
			    push (@xref,$stack{$lines[1]});
			}
			elsif ($#lines == 8 && $_ =~ /\s+(\d+)\s+/){
			    push (@results,"?/Z");
			    push (@xref,$stack{$lines[1]});
			}
		    }
		}
	    }
	}
	elsif(defined $words[$#words] && $words[$#words] eq "Refined"){
	    if (defined $groove){
		if ($groove eq "all"){
		    push (@results,"minor");
		    push (@results,"major");
		    push (@xref,"minor");
		    push (@xref,"major");
		}
		else{
		    push (@results,$groove);
		    push (@xref,$groove);
		}
		while(<RESULTS>){
		    if($_ =~ m/\*+/){
			last;
		    }
		    else{
			my @lines=split/\s+/,$_;
			if ($groove eq "all"){
			    push (@results,$lines[$param{'minor'}]);
			    push (@results,$lines[$param{'major'}]);
			    push (@xref,$bpref{$lines[1]});
			    push (@xref,$bpref{$lines[1]});
			}
			else{
			    push (@results,$lines[$param{$groove}]);
			    push (@xref,$bpref{$lines[1]});
			}
		    }
		}
	    }
	}
	elsif(defined $words[0] && $words[0] eq "Main"){
	    if (defined $torsion){
		my $tsn;
		if ($torsion ne "all"){
		    push (@results,$torsion);
		    push (@xref,$torsion);
		}
		else{
		    foreach $tsn (@tall){
			push (@results,$tsn);
			push (@xref,$tsn);
		    }
		}
		my $strandn;
		while(<RESULTS>){
		    if($_ =~ m/\*+/){
			last;
		    }
		    elsif ($torsion eq "all"){
			my @lines=split/\s+/,$_;
			if (defined $lines[0] && $lines[0] eq "Strand"){
			    $strandn=$lines[1];
			}
			elsif(defined $lines[1] && $lines[1] ne "base" && defined $strandn){
			    foreach $tsn (@tall){
				push (@results,$lines[$param{$tsn}]);
				push (@xref,$strand{$strandn}{$lines[1]});
			    }
			}
		    }
		    else{
			my @lines=split/\s+/,$_;
			if (defined $lines[0] && $lines[0] eq "Strand"){
			    $strandn=$lines[1];
			}
			elsif(defined $lines[1] && $lines[1] ne "base" && defined $strandn){
			    push (@results,$lines[$param{$torsion}]);
			    push (@xref,$strand{$strandn}{$lines[1]});
			}
		    }
		}
	    }
	}
	elsif(defined $words[0] && $words[0] eq "Sugar"){
	    if (defined $pucker){
		if ($pucker eq "ampl" || $pucker eq "phase" || $pucker eq "pucker"){
		    push (@results,$pucker);
		    push (@xref,$pucker);
		}
		elsif($pucker eq "both"){
		    push (@results,"amplitude");
		    push (@results,"phase");
		    push (@xref,"amplitude");
		    push (@xref,"phase");
		}
		elsif($pucker eq "all"){
		    push (@results,"amplitude");
		    push (@results,"phase");
		    push (@results,"pucker");
		    push (@xref,"amplitude");
		    push (@xref,"phase");
		    push (@xref,"pucker");
		}
		my $strandm;
		while(<RESULTS>){
		    if($_ =~ m/\*+/){
			last;
		    }
		    else{
			my @lines=split/\s+/,$_;
			if (defined $lines[0] && $lines[0] eq "Strand"){
			    $strandm=$lines[1];
			}
			elsif(defined $lines[1] && $lines[1] ne "base" && defined $strandm){
			    if ($pucker eq "ampl" || $pucker eq "phase" || $pucker eq "pucker"){
				push (@results,$lines[$param{$pucker}]);
				push (@xref,$strand{$strandm}{$lines[1]});
			    }
			    elsif($pucker eq "both"){
				push (@results,$lines[$param{ampl}]);
				push (@xref,$strand{$strandm}{$lines[1]});
				push (@results,$lines[$param{phase}]);
				push (@xref,$strand{$strandm}{$lines[1]});
			    }
			    elsif ($pucker eq "all"){
				push (@results,$lines[$param{ampl}]);
				push (@xref,$strand{$strandm}{$lines[1]});
				push (@results,$lines[$param{phase}]);
				push (@xref,$strand{$strandm}{$lines[1]});
				push (@results,$lines[$param{pucker}]);
				push (@xref,$strand{$strandm}{$lines[1]});
			    }
			}
		    }
		}
	    }
	}
	elsif(0){
	    #CHANGE THIS FOR OTHER PARAMETERS NOT PARSED ABOVE
	    #Note that the file is parsed in order from top to bottom
	    #so anything that comes before step, par, and helix should
	    #be added before those lines are read
	    #As well, you may need to append the reference arrays!
	}
    }
    close(RESULTS);
}

sub madbend (){
    #Parse through bp_step.par for base step parameters
    open (INP, "bp_step.par") || die "Cannot open file \"bp_step.par\"\n";
    while (<INP>){
	if ($_ =~ /\-?\d+\.\d+\s+\-?\d+\.\d+/){
	    my @r=split/\s+/,$_;
	    push (@tilt, $r[10]);
	    push (@roll, $r[11]);
	    push (@twist, $r[12]);
	}
    }
    close(INP);
    unlink ($show) if ($show eq "bp_step.par");
}

#########################
#                       #
#     Print Results     #
#                       #
#########################

open (OUT, ">$out") if (defined $out);

if ($madbend){
    if ($#tilt == $#roll && $#tilt == $#twist){
	my $basepairs=$#tilt;
	print "$basepairs\n";
	print OUT "$basepairs\n" if (defined $out);
    }
    for (my $j=1; $j<=$#tilt; $j++){ 
	#Always skip first base pair, no step info for single base pair!
	if ($tilt[$j] =~ /[0-9]+$/ && $roll[$j] =~ /[0-9]+$/ && $twist[$j] =~ /[0-9]+$/){
	    printf ("%7.2f\t%7.2f\t%7.2f\n", $tilt[$j],$roll[$j],$twist[$j]);
	    printf OUT ("%7.2f\t%7.2f\t%7.2f\n", $tilt[$j],$roll[$j],$twist[$j]) if (defined $out);
	}
	else{
	    print "   0.00\t   0.00\t   0.00\n";
	    print OUT "   0.00\t   0.00\t   0.00\n" if (defined $out);
	}
    }
    if (defined $ref){
	print STDERR "\n";
	open (REFOUT,">$ref");
	my $flag=undef;
	my $bstep=1;
	foreach (@xref){
	    print REFOUT "$_ ";
	    #STDERR Only below
	    $flag="tilt" if ($_ eq "tilt");
	    $flag=undef if ($_ eq "roll");
	    if (defined $flag && $_ ne "tilt"){
		print STDERR "($bstep)\t$_\n";
		$bstep++;
	    }
	    elsif (defined $flag && $_ eq "tilt"){
		print STDERR "Base Step (PDB RESID)\n";
	    }
	}
	close(REFOUT);
    }
}
else{
    foreach (@results){
	print "$_ ";
	print OUT "$_ " if (defined $out);
    }
    print "\n";
    print OUT "\n" if (defined $out);
    if (defined $ref){
	open (REFOUT,">$ref") || die "Cannot write to \"$ref\"\n";
	print REFOUT "BY PDB RESID (COLUMN)\n";
	my $column=1+$offset;
	foreach (@xref){
    print REFOUT "$_($column) ";
	    $column++;
	}
  close(REFOUT);
    }
}
